#include <kernel.h>
#include "kernel_cfg.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: httpd-fs.c,v 1.1 2006/06/07 09:13:08 adam Exp $
 */

#include "httpd.h"
#include "httpd-fs.h"
#include "httpd-fsdata.h"

#ifndef NULL
#define NULL 0
#endif /* NULL */

struct httpd_fsdata_file_noconst * HTTPD_FS_ROOT = 0;

/*-----------------------------------------------------------------------------------*/
static u8_t
httpd_fs_strcmp(const char *str1, const char *str2)
{
  u8_t i;
  i = 0;
 loop:

  if(str2[i] == 0 &&
    (str1[i] == 0 ||
     str1[i] == '\r' ||
     str1[i] == '\n')) {
    return 0;
  }

  if(str1[i] != str2[i]) {
    return 1;
  }


  ++i;
  goto loop;
}
/*-----------------------------------------------------------------------------------*/
int
httpd_fs_open(const char *name, struct httpd_fs_file *file)
{
  struct httpd_fsdata_file_noconst *f;
  struct httpd_fsdata_file_noconst *f_tail = (struct httpd_fsdata_file_noconst *)&HTTPD_FS_ROOT;

  for(f = (struct httpd_fsdata_file_noconst *)HTTPD_FS_ROOT;
      f != NULL;
      f = (struct httpd_fsdata_file_noconst *)f->next) {

    if(httpd_fs_strcmp(name, f->name) == 0) {
      if(f->data == 0)
      {
        if(E_OK != snd_dtq(DTQ_FF, (intptr_t)f->name))return 0;
        rcv_dtq(DTQ_FF_ACK, (intptr_t*)&f->data);
        rcv_dtq(DTQ_FF_ACK, (intptr_t*)&f->len);
      }
      file->data = f->data;
      file->len = f->len;
#if HTTPD_FS_STATISTICS
      ++(f->count);
#endif /* HTTPD_FS_STATISTICS */
      f->sec = 0;
      return 1;
    }
    f_tail = f;
  }
  //XgɖƂ̓J[h𒲂ׂ
  char nname[128];int n = 0;
  while(name[n] && name[n] != '\r' && name[n] != '\n'){nname[n] = name[n];n++;}nname[n] = '\0';
  if(E_OK == snd_dtq(DTQ_FF, (intptr_t)nname))
  {
    char *data;
    long len;
    rcv_dtq(DTQ_FF_ACK, (intptr_t*)&data);
    rcv_dtq(DTQ_FF_ACK, (intptr_t*)&len);
    if(data && len)
    {
      //J[hɂ΃Xǧ[ɒǉ
      f_tail->next = (struct httpd_fsdata_file *)calloc(sizeof(struct httpd_fsdata_file_noconst), 1);
      if(f_tail->next)
      {
        f_tail = (struct httpd_fsdata_file_noconst *)f_tail->next;
        f_tail->name = (char*)malloc(strlen(nname) + 1);
        if(f_tail->name)
        {
          sprintf(f_tail->name, "%s", nname);
        }
        file->data = f_tail->data = data;
        file->len = f_tail->len = len;
#if HTTPD_FS_STATISTICS
        f_tail->count = 1;
#endif /* HTTPD_FS_STATISTICS */
        return 1;
      }
    }
  }
  return 0;
}
/*-----------------------------------------------------------------------------------*/
void
httpd_fs_init(void)
{
}
/*-----------------------------------------------------------------------------------*/
#if HTTPD_FS_STATISTICS
u16_t httpd_fs_count
(char *name)
{
  struct httpd_fsdata_file_noconst *f;

  for(f = (struct httpd_fsdata_file_noconst *)HTTPD_FS_ROOT;
      f != NULL;
      f = (struct httpd_fsdata_file_noconst *)f->next) {

    if(httpd_fs_strcmp(name, f->name) == 0) {
      return f->count;
    }
  }
  return 0;
}
#endif /* HTTPD_FS_STATISTICS */
/*-----------------------------------------------------------------------------------*/
