#include <kernel.h>
#include "kernel_cfg.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "rtc_ctrl.h"
#include "task_calender.h"

#include "font.c"

enum rgb_color
{
	BLACK = 0,
	RED,
	GREEN,
	BLUE,
	YELLOW,
	CYAN,
	MAGENTA,
	WHITE,
};
static unsigned short rgb_color_table[] =
{
	0x0000,	//BLACK,
	0xf800,	//RED,
	0x07e0,	//GREEN,
	0x001f,	//BLUE,
	0xffe0,	//YELLOW,
	0x07ff,	//CYAN,
	0xf81f,	//MAGENTA,
	0xffff,	//WHITE,
};

#if 0
#define pset( x, y, col ); {*(unsigned short*)(0x04000000 + 320 * y + x) = rgb_color_table[col];}
#else
void pset(int x, int y, char color)
{
	unsigned short *p = (unsigned short *)0x04000000;
	*(p + 320 * y + x) = rgb_color_table[color];
}
#endif

//\
static void grph_cstr(int sx, int sy, char *string)
{
	int x, y, z = 0;

	for(z = 0; *string; z++)
	{
		char fcolor = *(string + 1);
		char bcolor = *(string + 2);

		for(y = 0; y < 10; y++)
		{
			char d = Font[*string][y];
			for(x = 0; x < 8; x++)
			{
				if(d & 1){
					pset(sx+x+z*8, sy+y, fcolor);
				}else{
					pset(sx+x+z*8, sy+y, bcolor);
				}
				d >>= 1;
			}
		}
		++string;
		++string;
		++string;
	}
	return;
}

//cQ{TCYŕ\
static void grph_cstr2(int sx, int sy, char *string)
{
	int x, y, z = 0;

	for(z = 0; *string; z++)
	{
		char fcolor = *(string + 1);
		char bcolor = *(string + 2);

		for(y = 0; y < 10; y++)
		{
			char d = Font[*string][y];
			for(x = 0; x < 8; x++)
			{
				if(d & 1){
					pset(sx + x * 2+0 + z * 8*2, sy + y * 2+0, fcolor);
					pset(sx + x * 2+1 + z * 8*2, sy + y * 2+0, fcolor);
					pset(sx + x * 2+0 + z * 8*2, sy + y * 2+1, fcolor);
					pset(sx + x * 2+1 + z * 8*2, sy + y * 2+1, fcolor);
				}else{
					pset(sx + x * 2+0 + z * 8*2, sy + y * 2+0, bcolor);
					pset(sx + x * 2+1 + z * 8*2, sy + y * 2+0, bcolor);
					pset(sx + x * 2+0 + z * 8*2, sy + y * 2+1, bcolor);
					pset(sx + x * 2+1 + z * 8*2, sy + y * 2+1, bcolor);
				}
				d >>= 1;
			}
		}
		++string;
		++string;
		++string;
	}
	return;
}

typedef struct{
	int year;
	int month;
	int mday;
	int hour;
	int min;
	int sec;
}T_TIME;

static void SetCStr(char* p1, char* p2, char fcolor, char bcolor)
{
	while(*p2)
	{
		*p1++ = *p2++;
		*p1++ = fcolor;
		*p1++ = bcolor;
	}
	*p1 = 0;
	return;
}

//uN//v\
static void DateDisp(int sx, int sy, T_TIME* time)
{
	char buf[11][3];
	int n;

	buf[0 ][0] = '0' + (unsigned char)((time->year/1000)%10);
	buf[1 ][0] = '0' + (unsigned char)((time->year/100)%10);
	buf[2 ][0] = '0' + (unsigned char)((time->year/10)%10);
	buf[3 ][0] = '0' + (unsigned char)((time->year/1)%10);
	buf[4 ][0] = '/';
	buf[5 ][0] = '0' + (unsigned char)((time->month/10)%10);
	buf[6 ][0] = '0' + (unsigned char)((time->month/1)%10);
	buf[7 ][0] = '/';
	buf[8 ][0] = '0' + (unsigned char)((time->mday/10)%10);
	buf[9 ][0] = '0' + (unsigned char)((time->mday/1)%10);
	buf[10][0] = '\0';

	for(n=0;n<11;n++)
	{
		buf[n][1] = BLACK;
		buf[n][2] = WHITE;
	}

	grph_cstr2(sx, sy, buf[0]);
	return;
}

//u//bv\
static void TimeDisp(int sx, int sy, T_TIME* time)
{
	char buf[9][3];
	int n;

	buf[0][0]='0'+(unsigned char)((time->hour/10)%10);
	buf[1][0]='0'+(unsigned char)((time->hour/1)%10);
	buf[2][0]=':';
	buf[3][0]='0'+(unsigned char)((time->min/10)%10);
	buf[4][0]='0'+(unsigned char)((time->min/1)%10);
	buf[5][0]=':';
	buf[6][0]='0'+(unsigned char)((time->sec/10)%10);
	buf[7][0]='0'+(unsigned char)((time->sec/1)%10);
	buf[8][0]='\0';

	for(n = 0; n < 9; n++)
	{
		buf[n][1] = BLACK;
		buf[n][2] = WHITE;
	}

	grph_cstr2(sx, sy, buf[0]);
	return;
}

//cQ{TCYŐ\
static void NumDisp2(int sx, int sy, unsigned char num, char fcolor, char bcolor)
{
	char buf[2][3];

	buf[0][0]='0' + num;
	buf[1][0]='\0';

	buf[0][1] = fcolor;
	buf[0][2] = bcolor;

	grph_cstr2(sx, sy, buf[0]);
	return;
}

static int HolidayCheck(int year, int month, int mday, int week)
{
	int n;
	const int holiday[][2]={
		{1,1},		//U
		{2,11},		//LO̓
		{3,20},		//t̓
		{4,29},		//a̓
		{5,3},		//@LO
		{5,4},		//݂ǂ̓
		{5,5},		//ǂ̓
		{9,22},		//H̓
		{11,3},		//̓
		{11,23},	//ΘJӂ̓
		{12,23},	//Vca
		{0,0}
	};

	for(n = 0; holiday[n][0]; n++)
	{
		if(holiday[n][0] == month
		&& holiday[n][1] == mday)
		{
			return 1;
		}
	}

	if(week == 1)
	{
		//nbs[}f[
		if(month == 1  && mday > 7*(2-1) && mday <= 7*2)return 1;	//l̓
		if(month == 7  && mday > 7*(3-1) && mday <= 7*3)return 1;	//C̓
		if(month == 9  && mday > 7*(3-1) && mday <= 7*3)return 1;	//hV̓
		if(month == 10 && mday > 7*(2-1) && mday <= 7*2)return 1;	//̈̓

		//U֋x
		for(n = 0; holiday[n][0]; n++)
		{
			if(holiday[n][0] == month
			&& holiday[n][1] == mday - 1)
			{
				return 1;
			}
		}
	}
	return 0;
}

//j̎Zo
static int get_week(int year, int month, int mday)
{
	int n;
	long w = 2;

	for(n = 0; n < year; ++n)
	{
		w += 365;
		if((n % 4 == 0 && n % 100 != 0) || n % 1000 == 0)++w;
	}

	for(n = 1; n < month; ++n)
	{
		switch(n)
		{
			case 2:w += 28;if((year % 4 == 0 && year % 100 != 0) || year % 1000 == 0)++w;break;
			case 4:
			case 6:
			case 9:
			case 11:w += 30;break;
			default:w += 31;
		}
	}
	return (w + mday - 1) % 7;
}

static void WeekDisp(int sx, int sy)
{
	char buf[41][3];

	SetCStr(buf[0 ], " ",   BLACK, WHITE);
	SetCStr(buf[1 ], "SUN", RED,   WHITE); SetCStr(buf[4 ], "   ", BLACK, WHITE);
	SetCStr(buf[7 ], "MON", BLACK, WHITE); SetCStr(buf[10], "   ", BLACK, WHITE);
	SetCStr(buf[13], "TUE", BLACK, WHITE); SetCStr(buf[16], "   ", BLACK, WHITE);
	SetCStr(buf[19], "WED", BLACK, WHITE); SetCStr(buf[22], "   ", BLACK, WHITE);
	SetCStr(buf[25], "THU", BLACK, WHITE); SetCStr(buf[28], "   ", BLACK, WHITE);
	SetCStr(buf[31], "FRI", BLACK, WHITE); SetCStr(buf[34], "   ", BLACK, WHITE);
	SetCStr(buf[37], "SAT", BLUE,  WHITE);
	buf[40][0]='\0';

	grph_cstr(sx, sy, buf[0]);
	return;
}

static void TableDisp(int sx, int sy, int year, int month, int mday)
{
	char buf[21][3];
	int x, y, d, dmax;
	int w = get_week(year, month, 1);

	switch(month)
	{
		case 2:dmax = 28;if((year % 4 == 0 && year % 100 != 0) || year % 1000 == 0)++dmax;break;
		case 4:
		case 6:
		case 9:
		case 11:dmax = 30;break;
		default:dmax = 31;
	}
	d = 1 - w;
	for(y = 0; y < 6; ++y)
	{
		for(x = 0; x < 7; ++x)
		{
			if(d > 0 && d <= dmax)
			{
				int holiday = HolidayCheck(year, month, d, x);

				if(d < 10)
				{
					buf[x * 3+0][0] = ' ';
				}
				else
				{
					buf[x * 3+0][0] = '0' + (unsigned char)((d / 10) % 10);
				}

				if(holiday == 3)
				{
					buf[x * 3+0][1] = YELLOW;	//
					buf[x * 3+1][1] = YELLOW;	//
				}
				else
				if(holiday == 2)
				{
					buf[x * 3+0][1] = MAGENTA;	//x
					buf[x * 3+1][1] = MAGENTA;	//x
				}
				else
				if(holiday == 1)
				{
					buf[x * 3+0][1] = RED;		//j
					buf[x * 3+1][1] = RED;		//j
				}
				else
				if(x == 6)
				{
					buf[x * 3+0][1] = BLUE	;	//yj
					buf[x * 3+1][1] = BLUE	;	//yj
				}
				else
				if(x == 0)
				{
					buf[x * 3+0][1] = RED;		//j
					buf[x * 3+1][1] = RED;		//j
				}
				else
				{
					buf[x * 3+0][1] = BLACK;	//
					buf[x * 3+1][1] = BLACK;	//
				}

				if(mday == d)
				{
					buf[x * 3+0][2] = GREEN;	//
					buf[x * 3+1][2] = GREEN;	//
				}
				else
				{
					buf[x * 3+0][2] = WHITE;
					buf[x * 3+1][2] = WHITE;
				}
				buf[x * 3+1][0]='0' + (unsigned char)((d / 1) % 10);
			}
			else
			{
				buf[x * 3+0][0] = ' ';
				buf[x * 3+0][1] = WHITE;
				buf[x * 3+0][2] = WHITE;

				buf[x * 3+1][0] = ' ';
				buf[x * 3+1][1] = WHITE;
				buf[x * 3+1][2] = WHITE;
			}
			buf[x * 3+2][0] = ' ';
			buf[x * 3+2][1] = WHITE;
			buf[x * 3+2][2] = WHITE;
			++d;
		}
		if(y == 5)
		{
			buf[5][0] = '\0';
		}
		else
		{
			buf[20][0] = '\0';
		}
		grph_cstr2(sx, sy + 25 * y, buf[0]);
	}
	return;
}

static void disp_clear(void)
{
	unsigned short *p = (unsigned short *)0x04000000;
	int x,y;

	for(y = 0; y < 240; y++)
	{
		for(x = 0; x < 320; x++)
		{
				*p++ = 0xffff;
		}
	}
	return;
}

void task_calender(intptr_t exinf)
{
	T_TIME time = {0,0,0,0,0,0};
	T_TIME tbck = {0,0,0,0,0,0};

	int sync_cycle = 8 * 3600;		//WԖ
	const char ntp_url[] = "time-a.nist.gov";
	char ntp[128];
	int count = 10;		//10bɎ

	disp_clear();
	init_rtc();

	while(1)
	{
		unsigned int data;
		rcv_dtq(DTQ_CALENDER, (intptr_t*)&data);

		if(data == 0)
		{
			if(count-- == 0)
			{
				strcpy(ntp, "NTP");
				strcat(ntp, ntp_url);
				snd_dtq(DTQ_UIP, (intptr_t)ntp);
				count = 10;
			}
		}
		else
		{
			count = sync_cycle;

			printf("task_calender/rcv_dtq = 0x%08x\n", data);
			data += 60*60*9;	//GMTƂ̍

			time.year	= 1900;
			time.month	= 1;
			time.mday	= 1;
			time.hour	= (data % (60*60*24)) / (60*60);
			time.min	= (data % (60*60)) / 60;
			time.sec	= data % 60;

			for( ; data >= 60*60*24; data -= 60*60*24)
			{
				int leap = (((time.year % 4 == 0) && (time.year % 100 != 0)) || (time.year % 400 == 0)) ? 1 : 0;	//[N
				const int day[2][1+12] =
				{
					//  1 2 3 4 5 6 7 8 9 101112
					{0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
					{0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
				};

				if(++time.mday > day[leap][time.month])
				{
					time.mday = 1;
					if(++time.month > 12)
					{
						time.month = 1;
						time.year++;
					}
				}
			}
			rtc_set(time.year, time.month, time.mday, time.hour, time.min, time.sec);
			printf("task_calender/%d/%02d/%02d %02d:%02d:%02d\n", time.year, time.month, time.mday, time.hour, time.min, time.sec);
		}

		rtc_get(&time.year, &time.month, &time.mday, &time.hour, &time.min, &time.sec);
		if(tbck.year != time.year || tbck.month != time.month || tbck.mday != time.mday)
		{
			DateDisp(10, 5, &time);
			TimeDisp(10+11*8*2, 5, &time);
			WeekDisp(0, 35+15);
			TableDisp(1, 35+15+15, time.year, time.month, time.mday);
		}
		else
		{
			if(tbck.sec % 10 != time.sec % 10)
			{
				NumDisp2(10+18*8*2, 5, time.sec % 10, BLACK, WHITE);

				if(tbck.sec / 10 != time.sec / 10)
				{
					NumDisp2(10+17*8*2, 5, time.sec / 10, BLACK, WHITE);

					if(tbck.min % 10 != time.min % 10)
					{
						NumDisp2(10+15*8*2, 5, time.min % 10, BLACK, WHITE);

						if(tbck.min / 10 != time.min / 10)
						{
							NumDisp2(10+14*8*2, 5, time.min / 10, BLACK, WHITE);

							if(tbck.hour % 10 != time.hour % 10)
							{
								NumDisp2(10+12*8*2, 5, time.hour % 10, BLACK, WHITE);

								if(tbck.hour / 10 != time.hour / 10)
								{
									NumDisp2(10+11*8*2, 5, time.hour / 10, BLACK, WHITE);
								}
							}
						}
					}
				}
			}
		}
		tbck = time;
	}
}

void DisplayIPAddress(const unsigned short ipaddr[])
{
    char dis_buf[] = "xxx.xxx.xxx.xxx\0";
    char *p = (char *)ipaddr;

    sprintf(dis_buf, "%d.%d.%d.%d\0", *p, *(p+1), *(p+2), *(p+3));

    char cstr_buf[sizeof(dis_buf) * 3];
    SetCStr(cstr_buf, dis_buf, BLACK, WHITE);
    grph_cstr(1, 239-10, cstr_buf);

    return;
}
