#include <kernel.h>
#include "syssvc/serial.h"
#include "kernel_cfg.h"

#include "kernel_impl.h"
#include <sil.h>
#include "frk_rx62n.h"
#include "rx600/rx600_uart.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "iodefine.h"
#include "webcam.h"
#include "task_camera.h"

#define ALARM_TIME (60*1000)
static int power_saving;

static void camera_Reset(void)
{
	unsigned char c[256];

	//Reset
	c[0] = 0x56;
	c[1] = 0x00;
	c[2] = 0x26;
	c[3] = 0x00;

	printf("task_camera/send Reset\n");
	serial_wri_dat((ID)CAMERA_PORTID, (const char_t *)c, (uint_t)4);
	serial_rea_dat((ID)CAMERA_PORTID, (char_t *)c, (uint_t)5);
	printf("task_camera/receive Reset ack %x,%x,%x,%x,%x\n", c[0],c[1],c[2],c[3],c[4]);
	serial_rea_dat((ID)CAMERA_PORTID, (char_t *)c, (uint_t)0x47 - 5);
	printf("task_camera/receive message \n\n%s\n", c);

	dly_tsk(2000);
	return;
}

static void camera_Power_Saving(int mode)
{
	unsigned char c[256];

	//Power_Saving
	c[0] = 0x56;
	c[1] = 0x00;
	c[2] = 0x3e;
	c[3] = 0x03;
	c[4] = 0x00;
	c[5] = 0x01;
	c[6] = mode ? 0x01 : 0x00;

	serial_wri_dat((ID)CAMERA_PORTID, (const char_t *)c, (uint_t)7);
	serial_rea_dat((ID)CAMERA_PORTID, (char_t *)c, (uint_t)5);
	printf("task_camera/receive Power_Saving ack %x,%x,%x,%x,%x\n", c[0],c[1],c[2],c[3],c[4]);
	return;
}

static unsigned char* camera_Take_picture(int* size)
{
	unsigned char c[256];
	unsigned char* buf;

	printf("task_camera/ready\n");
	if(power_saving)camera_Power_Saving(power_saving = 0);

	//Take_picture
	c[0] = 0x56;
	c[1] = 0x00;
	c[2] = 0x36;
	c[3] = 0x01;
	c[4] = 0x00;
	serial_wri_dat((ID)CAMERA_PORTID, (const char_t *)c, (uint_t)5);
	serial_rea_dat((ID)CAMERA_PORTID, (char_t *)c, (uint_t)5);
	printf("task_camera/receive Take_picture ack %x,%x,%x,%x,%x\n", c[0],c[1],c[2],c[3],c[4]);

	//Read_JPEG_file_size
	c[0] = 0x56;
	c[1] = 0x00;
	c[2] = 0x34;
	c[3] = 0x01;
	c[4] = 0x00;
	serial_wri_dat((ID)CAMERA_PORTID, (const char_t *)c, (uint_t)5);
	serial_rea_dat((ID)CAMERA_PORTID, (char_t *)c, (uint_t)9);
	printf("task_camera/receive Read_JPEG_file_size ack %x,%x,%x,%x,%x,%x,%x,%x,%x\n", c[0],c[1],c[2],c[3],c[4],c[5],c[6],c[7],c[8]);

	unsigned char kh = c[7];
	unsigned char kl = c[8];

	*size = kh *0x100 + kl;
	buf = malloc(*size);
	printf("task_camera/malloc 0x%x, %d\n", buf, *size);

	if(buf)
	{
		//Read_JPEG_file_content
		c[0] = 0x56;
		c[1] = 0x00;
		c[2] = 0x32;
		c[3] = 0x0c;
		c[4] = 0x00;
		c[5] = 0x0a;
		c[6] = 0x00;
		c[7] = 0x00;
		c[8] = 0x00;	//MH
		c[9] = 0x00;	//ML
		c[10] = 0x00;
		c[11] = 0x00;
		c[12] = kh;		//KH
		c[13] = kl;		//KL
		c[14] = 0x00;	//XX
		c[15] = 0x0a;	//XX
		serial_wri_dat((ID)CAMERA_PORTID, (const char_t *)c, (uint_t)16);
		serial_rea_dat((ID)CAMERA_PORTID, (char_t *)c, (uint_t)(5));
		printf("task_camera/receive header %x,%x,%x,%x,%x\n", c[0],c[1],c[2],c[3],c[4]);

		printf("task_camera/wait\n");
		serial_rea_dat((ID)CAMERA_PORTID, (char_t *)(buf), (uint_t)(*size));
		printf("task_camera/receive Read_JPEG_file_content\n");
		serial_rea_dat((ID)CAMERA_PORTID, (char_t *)c, (uint_t)5);
		printf("task_camera/receive Read_JPEG_file_content end %x,%x,%x,%x,%x\n", c[0],c[1],c[2],c[3],c[4]);

		//Stop_taking_picture
		c[0] = 0x56;
		c[1] = 0x00;
		c[2] = 0x36;
		c[3] = 0x01;
		c[4] = 0x03;
		serial_wri_dat((ID)CAMERA_PORTID, (const char_t *)c, (uint_t)5);
		serial_rea_dat((ID)CAMERA_PORTID, (char_t *)c, (uint_t)5);
		printf("task_camera/receive Stop_taking_picture ack %x,%x,%x,%x,%x\n", c[0],c[1],c[2],c[3],c[4]);
	}
	return buf;
}

static void camera_Image_size(int mode)
{
	unsigned char c[256];
	unsigned char size[] = {0x22, 0x11, 0x00};	//QQVGA, QVGA, VGA

	if(power_saving)camera_Power_Saving(power_saving = 0);

	//Image_size
	c[0] = 0x56;
	c[1] = 0x00;
	c[2] = 0x31;
	c[3] = 0x05;
	c[4] = 0x04;
	c[5] = 0x01;
	c[6] = 0x00;
	c[7] = 0x19;
	c[8] = size[mode];

	serial_wri_dat((ID)CAMERA_PORTID, (const char_t *)c, (uint_t)9);
	serial_rea_dat((ID)CAMERA_PORTID, (char_t *)c, (uint_t)5);
	printf("task_camera/receive Image_size ack %x,%x,%x,%x,%x\n", c[0],c[1],c[2],c[3],c[4]);
	return;
}

void task_camera(intptr_t exinf)
{
	IOPORT.PFFSCI.BIT.SCI2S = 1;
	MSTP_SCI2 = 1;

	dly_tsk(2000);

	IOPORT.PFFSCI.BIT.SCI2S = 0;
	MSTP_SCI2 = 0;
	PORT1.ICR.BIT.B2 = 1;

	serial_opn_por(CAMERA_PORTID);
	serial_ctl_por(CAMERA_PORTID, IOCTL_NULL);

	int size = 0;
	power_saving = 1;
	camera_Image_size(size);
	camera_Reset();
	sta_alm(ALM_CAMERA_POWER_SAVING, ALARM_TIME);

	while(1)
	{
		unsigned int data;
		rcv_dtq(DTQ_CAMERA, (intptr_t*)&data);
		printf("task_camera/rcv_dtq = %s\n", data);

		if(strstr((const char *)data, "RESET"))
		{
			camera_Reset();
			snd_dtq(DTQ_CAMERA_ACK, (intptr_t)0);
		}
		else
		if(strstr((const char *)data, "TAKE_PICTURE"))
		{
			unsigned char* buf;
			int size;
			buf = camera_Take_picture(&size);
			printf("task_camera/Take_picture 0x%x, %d\n", buf, size);
			snd_dtq(DTQ_CAMERA_ACK2, (intptr_t)buf);
			snd_dtq(DTQ_CAMERA_ACK2, (intptr_t)size);
		}
		else
		if(strstr((const char *)data, "QQVGA"))
		{
			if(size != 0)
			{
				size = 0;
				camera_Image_size(size);
				camera_Reset();
			}
			snd_dtq(DTQ_CAMERA_ACK3, (intptr_t)"QQVGA");
			printf("task_camera/Image_size : QQVGA\n");
		}
		else
		if(strstr((const char *)data, "QVGA"))
		{
			if(size != 1)
			{
				size = 1;
				camera_Image_size(size);
				camera_Reset();
			}
			snd_dtq(DTQ_CAMERA_ACK3, (intptr_t)"QVGA");
			printf("task_camera/Image_size : QVGA\n");
		}
		else
		if(strstr((const char *)data, "VGA"))
		{
			if(size != 2)
			{
				size = 2;
				camera_Image_size(size);
				camera_Reset();
			}
			snd_dtq(DTQ_CAMERA_ACK3, (intptr_t)"VGA");
			printf("task_camera/Image_size : VGA\n");
		}

		if(strstr((char*)data, "GET_SIZE"))
		{
			snd_dtq(DTQ_CAMERA_ACK, (intptr_t)size);
		}

		if(strstr((const char *)data, "ENTERING_POWER_SAVING"))
		{
			if(!power_saving)
			{
				camera_Power_Saving(power_saving = 1);
			}
			printf("task_camera/Power_Saving = %d\n", power_saving);
			continue;
		}

		sta_alm(ALM_CAMERA_POWER_SAVING, ALARM_TIME);
	}
}

void alm_camera_Power_Saving(intptr_t exinf)
{
	ipsnd_dtq(DTQ_CAMERA, (intptr_t)"ENTERING_POWER_SAVING");
	return;
}
