#include <kernel.h>
#include "kernel_cfg.h"

#include <stdio.h>
#include <string.h>

#include "mtu2_ctrl.h"
#include "task_servo.h"

//͈͂̐ݒ
#define WMIN -90
#define WMAX +90

#define DELAY_TIME_SHORT	(1)
#define DELAY_TIME_LONG		(600)
#define ALARM_TIME			(3000)

static int pan, tilt, home_pan, home_tilt;

static short set_pan(short v)
{
	v = (v > WMAX) ? WMAX : v;
	v = (v < WMIN) ? WMIN : v;
	return v;
}

static short set_tilt(short v)
{
	v = (v > WMAX) ? WMAX : v;
	v = (v < WMIN) ? WMIN : v;
	return v;
}

static void home_set(void)
{
	mtu3_set(pan = set_pan(WMIN));
	dly_tsk(DELAY_TIME_LONG);

	mtu4_set(tilt = set_tilt(WMIN));
	dly_tsk(DELAY_TIME_LONG);

	mtu3_set(pan = set_pan(home_pan));
	dly_tsk(DELAY_TIME_LONG);

	mtu4_set(tilt = set_tilt(home_tilt));
	dly_tsk(DELAY_TIME_LONG);

	sta_alm(ALM_SERVO_OFF, ALARM_TIME);
	return;
}

void task_servo(intptr_t exinf)
{
	int step = 30;

	pan = tilt = 0;
	home_pan = 0;
	home_tilt = 0;

	init_mtu3();
	init_mtu4();
	home_set();

	while(1)
	{
		unsigned int data;
		rcv_dtq(DTQ_SERVO, (intptr_t*)&data);
		printf("task_servo/rcv_dtq = %s\n", data);

		if(strstr((char*)data, "SET"))
		{
			home_pan = pan;
			home_tilt = tilt;
			printf("task_servo/home set pan = %d, tilt = %d\n", home_pan, home_tilt);
		}
		else
		if(strstr((char*)data, "HOME"))
		{
			home_set();
		}
		else
		{
			if(strstr((char*)data, "LEFT"))
			{
				mtu3_set(pan = set_pan(pan - step));
				mtu4_set(tilt = set_tilt(tilt));
				dly_tsk(DELAY_TIME_LONG);
				sta_alm(ALM_SERVO_OFF, ALARM_TIME);
			}
			else
			if(strstr((char*)data, "RIGHT"))
			{
				mtu3_set(pan = set_pan(pan + step));
				mtu4_set(tilt = set_tilt(tilt));
				dly_tsk(DELAY_TIME_LONG);
				sta_alm(ALM_SERVO_OFF, ALARM_TIME);
			}
			else
			if(strstr((char*)data, "UP"))
			{
				mtu3_set(pan = set_pan(pan));
				mtu4_set(tilt = set_tilt(tilt + step));
				dly_tsk(DELAY_TIME_LONG);
				sta_alm(ALM_SERVO_OFF, ALARM_TIME);
			}
			else
			if(strstr((char*)data, "DOWN"))
			{
				mtu3_set(pan = set_pan(pan));
				mtu4_set(tilt = set_tilt(tilt - step));
				dly_tsk(DELAY_TIME_LONG);
				sta_alm(ALM_SERVO_OFF, ALARM_TIME);
			}
		}

		if(strstr((char*)data, "STEP30"))
		{
			step = 30;
		}
		else
		if(strstr((char*)data, "STEP10"))
		{
			step = 10;
		}
		else
		if(strstr((char*)data, "STEP01"))
		{
			step = 1;
		}

		if(strstr((char*)data, "GET_PAN"))
		{
			snd_dtq(DTQ_SERVO_ACK, (intptr_t)pan);
			continue;
		}
		if(strstr((char*)data, "GET_TILT"))
		{
			snd_dtq(DTQ_SERVO_ACK, (intptr_t)tilt);
			continue;
		}
		if(strstr((char*)data, "GET_STEP"))
		{
			snd_dtq(DTQ_SERVO_ACK, (intptr_t)step);
			continue;
		}

		if(strstr((char*)data, "SERVO_OFF"))
		{
			mtu34_stop();
			printf("task_servo/servo off\n");
			continue;
		}

		printf("task_servo/pan = %d, tilt = %d\n", pan, tilt);
		snd_dtq(DTQ_SERVO_ACK, (intptr_t)0);
	}
}

void alm_servo_off(intptr_t exinf)
{
	ipsnd_dtq(DTQ_SERVO, (intptr_t)"SERVO_OFF");
	return;
}
