#include <kernel.h>
#include "kernel_cfg.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "iodefine.h"
#include "typedefine.h"

#include "uip.h"
#include "uiplib.h"
#include "uip_arp.h"
// Renesas -- #include "network-device.h"
#include "r_ether.h"
#include "httpd.h"
#include "resolv.h"
#include "timer.h"
#include "user-app.h"

#include "task_uip.h"

#define BUF ((struct uip_eth_hdr *)&uip_buf[0])

#ifdef USE_FIX_IPADDR
extern const u8_t my_ip[4];
extern const u8_t my_netmask[4];
extern const u8_t my_default_router[4];
extern const u8_t my_dnsaddr[4];
#endif
extern const struct uip_eth_addr my_mac;

static int req_ntp = 0;

void task_uip(intptr_t exinf)
{
  int i;
  #ifdef USE_FIX_IPADDR
  uip_ipaddr_t ipaddr;
  #endif
  struct timer periodic_timer, arp_timer;
  uint32_t ch = 0;

  printf("task_uip/uIP Demo\n");

  timer_set(&periodic_timer, CLOCK_SECOND / 2);
  timer_set(&arp_timer, CLOCK_SECOND * 10);

  // Renesas -- network_device_init();
  /* Wait until Ether device initailize succesfully. 
     Make sure Ethernet cable is plugged in. */ 
  while (R_ETHER_ERROR == R_Ether_Open(ch, (uint8_t*)&my_mac.addr[0]));

  // Renesas ++ set Ethernet address
  uip_setethaddr(my_mac);
  uip_init();

  #ifdef USE_FIX_IPADDR
  uip_ipaddr(ipaddr, my_ip[0], my_ip[1], my_ip[2], my_ip[3]);
  uip_sethostaddr(ipaddr);
  uip_ipaddr(ipaddr, my_netmask[0], my_netmask[1], my_netmask[2], my_netmask[3]);
  uip_setnetmask(ipaddr);
  uip_ipaddr(ipaddr, my_default_router[0], my_default_router[1], my_default_router[2], my_default_router[3]);
  uip_setdraddr(ipaddr);
  uip_ipaddr(ipaddr, my_dnsaddr[0], my_dnsaddr[1], my_dnsaddr[2], my_dnsaddr[3]);
  resolv_conf(ipaddr);
  #else
  dhcpc_init(&my_mac.addr[0], 6);
  #endif
  httpd_init();
  resolv_init();

  while (1)
  {
    // Renesas -- uip_len = network_device_read();
    uip_len = R_Ether_Read(ch, (void *)uip_buf);
    if (uip_len > 0)
    {
      if (BUF->type == htons(UIP_ETHTYPE_IP))
      {
        uip_arp_ipin();
        uip_input();
        /* If the above function invocation resulted in data that
           should be sent out on the network, the global variable
           uip_len is set to a value > 0. */
        if (uip_len > 0)
        {
          uip_arp_out();
          // Renesas -- network_device_send();
          R_Ether_Write(ch, (void *)uip_buf, (uint32_t)uip_len);
        }
      }
      else if (BUF->type == htons(UIP_ETHTYPE_ARP))
      {
        uip_arp_arpin();
        /* If the above function invocation resulted in data that
           should be sent out on the network, the global variable
           uip_len is set to a value > 0. */
        if (uip_len > 0)
        {
          // Renesas -- network_device_send();
          R_Ether_Write(ch, (void *)uip_buf, (uint32_t)uip_len);
        }
      }

    }
    else if (timer_expired(&periodic_timer))
    {
      timer_reset(&periodic_timer);
      for (i = 0; i < UIP_CONNS; i++)
      {
        uip_periodic(i);
        /* If the above function invocation resulted in data that
           should be sent out on the network, the global variable
           uip_len is set to a value > 0. */
        if (uip_len > 0)
        {
          uip_arp_out();
          // Renesas -- network_device_send();
          R_Ether_Write(ch, (void *)uip_buf, (uint32_t)uip_len);
        }
      }

#if UIP_UDP
      for (i = 0; i < UIP_UDP_CONNS; i++)
      {
        uip_udp_periodic(i);
        /* If the above function invocation resulted in data that
           should be sent out on the network, the global variable
           uip_len is set to a value > 0. */
        if (uip_len > 0)
        {
          uip_arp_out();
          // Renesas -- network_device_send();
          R_Ether_Write(ch, (void *)uip_buf, (uint32_t)uip_len);
        }
      }
#endif /* UIP_UDP */

      /* Call the ARP timer function every 10 seconds. */
      if (timer_expired(&arp_timer))
      {
        timer_reset(&arp_timer);
        uip_arp_timer();
      }
    }

    // Insert user aplications here.
    unsigned int data;
    if(E_TMOUT != prcv_dtq(DTQ_UIP, (intptr_t*)&data))
    {
      printf("task_uip/rcv_dtq = %s\n", data);

      if(strstr((char*)data, "DNS"))
      {
        resolv_query((char*)data + 3);
      }
      else
      if(strstr((char*)data, "NTP"))
      {
        uip_ipaddr_t *ipaddr;
        static uip_ipaddr_t addr;

        ipaddr = &addr;
        if(uiplib_ipaddrconv((char*)data + 3, (unsigned char *)addr) == 0)
        {
          ipaddr = (uip_ipaddr_t *)resolv_lookup((char*)data + 3);
          if(ipaddr == NULL)
          {
            memcpy((char*)data, "DNS", 3);
            snd_dtq(DTQ_UIP, (intptr_t)data);
          }
          else
          {
            uip_udp_new(ipaddr, (u16_t)HTONS(123));
            req_ntp = 1;
          }
        }
        else
        {
          uip_udp_new(ipaddr, (u16_t)HTONS(123));
          req_ntp = 1;
        }
      }
      else
      {
        resolv_query((char*)data);
      }
    }
  }
  return;
}
/*---------------------------------------------------------------------------*/
void
uip_log(char *m)
{
  printf("uIP log message: %s\n", m);
}

void
resolv_found(char *name, u16_t *ipaddr)
{
  if(ipaddr == NULL) {
    printf("Host '%s' not found.\n", name);
  } else {
    printf("Found name '%s' = %d.%d.%d.%d\n", name,
	   htons(ipaddr[0]) >> 8,
	   htons(ipaddr[0]) & 0xff,
	   htons(ipaddr[1]) >> 8,
	   htons(ipaddr[1]) & 0xff);
    /*    webclient_get("www.sics.se", 80, "/~adam/uip");*/
    static char ntp_addr[32];
    sprintf(ntp_addr, "NTP%d.%d.%d.%d\0",
	   htons(ipaddr[0]) >> 8,
	   htons(ipaddr[0]) & 0xff,
	   htons(ipaddr[1]) >> 8,
	   htons(ipaddr[1]) & 0xff);
    snd_dtq(DTQ_UIP, (intptr_t)ntp_addr);
  }
}

void
sntp_appcall(void)
{
  if(uip_udp_conn->rport == HTONS(123))
  {
    if(uip_poll())
    {
      if(req_ntp)
      {
        memset((char *)uip_appdata, 0, (4*4 + 4*8));
        *(unsigned char *)uip_appdata = 0xd9;
        uip_udp_send((unsigned char)(4*4 + 4*8));
        req_ntp = 0;
      }
    }
    if(uip_newdata())
    {
      unsigned char* p = (unsigned char*)uip_appdata + 0x28;
      snd_dtq(DTQ_CALENDER, (intptr_t)((*(p + 0) << 24) | (*(p + 1) << 16) | (*(p + 2) << 8) | (*(p + 3) << 0)));
    }
  }
  else
  if(uip_udp_conn->rport == HTONS(53))
  {
    resolv_appcall();
  }
  else
  {
    dhcpc_appcall();
  }
}
